#![cfg_attr(nightly, feature(never_type))] 

#![allow(dead_code)]

use cfg_if::cfg_if;
use lazy_static::lazy_static;
use futures::prelude::*;

#[allow(unused_imports)]
use color_eyre::{
    eyre::{
	self,
	eyre,
	WrapErr as _,
    },
    SectionExt as _, Help as _,
};
use std::{
    sync::Arc,
};

mod ext;
use ext::*;

mod util;
mod args;
mod config;
mod resolve;
mod dedup;

cfg_if!{
    if #[cfg(nightly)] {
	type Never = !;
    } else {
	type Never = std::convert::Infallible;
    }
}

type Pointer = *const Never;

fn install() -> eyre::Result<()>
{
    color_eyre::install()?;

    Ok(())
}

async fn process(config: Arc<config::Config>, file: String)
{
    println!(" -> {:?}", file);
    let dbdir = resolve::mangle_path(&config, &file);
    println!("Database path for this file {:?}", dbdir);
    println!("Demangle: {:?}", resolve::demangle_path(&dbdir).await);
}

async fn begin() -> eyre::Result<i32>
{
    install()?;
    let config = Arc::new(config::Config::default());
    if args::process(|file| {
	let config = Arc::clone(&config);
	process(config, file)
    }).await
        .wrap_err(eyre!("One or more child workers failed to complete successfully"))?
	.len() == 0
    {
	args::usage();
    }
    
    Ok(0)
}

#[tokio::main]
async fn main() -> eyre::Result<()>
{
    std::process::exit(match begin().await {
	Ok(0) => return Ok(()),
	Err(err) => {
	    eprintln!("\nexited with error: {}", err);
	    return Err(err)
		.wrap_err(eyre!("Fatal error"));
	},
	Ok(v) => v
    })
}
