//! Videl path resolution
use super::*;
use std::{
    path::{
	Path,
	PathBuf,
    },
    collections::HashMap,
    fmt,
    error,
};
use std::os::unix::ffi::{OsStrExt, OsStringExt};

#[cfg(not(feature="fast-pathnames"))] 
fn compute_hash_string(from: impl AsRef<[u8]>) -> String
{
    use sha2::{Digest, Sha256};
    let mut sha2 = Sha256::new();
    sha2.update(from.as_ref());
    let output = sha2.finalize();
    output.into_iter().hex().collect()
}

lazy_static!{
    static ref B64_TO: HashMap<char, char> = {
	let mut table = HashMap::new();
	table.insert('/', '-'); //cannot appear in file paths, to
	table
    };
    static ref B64_FROM: HashMap<char, char> = {
	B64_TO.iter().map(|(&x,&y)| (y,x)).collect()
    };
}

fn replace_base64_to(string: impl AsRef<str>) -> String
{
    string.as_ref().chars().replace_with(&B64_TO).collect()
}

fn replace_base64_from(string: impl AsRef<str>) -> String
{
    string.as_ref().chars().replace_with(&B64_FROM).collect()
}

/// Resolve the database path for a certain file
pub fn mangle_path(config: &config::Config, path: impl AsRef<Path>) -> PathBuf
{
    cfg_if!{
	if #[cfg(feature="fast-pathnames")] {
	    config.base_dir.join(replace_base64_to(base64::encode(path.as_ref().as_os_str().as_bytes())))
	} else {
	    config.base_dir.join(compute_hash_string(path.as_ref().as_os_str().as_bytes()))
	}
    }
}

#[derive(Debug)]
pub struct ResolutionError;

impl error::Error for ResolutionError{}
impl fmt::Display for ResolutionError
{
    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result
    {
	write!(f, "database path was in an invalid format")
    }
}

/// Find the original path from a database one
pub async fn demangle_path(path: impl AsRef<Path>) -> Result<PathBuf, ResolutionError>
{
    cfg_if! {
	if #[cfg(feature="fast-pathnames")] {
	    let part = path.as_ref().file_name().ok_or(ResolutionError)?; //get the base64 encoded part
	    let part = replace_base64_from(part.to_str().ok_or(ResolutionError)?); //replace characters back
	    let bytes = base64::decode(part).map_err(|_| ResolutionError)?;
	    
	    Ok(std::ffi::OsString::from_vec(bytes).into())
	} else {
	    //TODO: Look up in `path/metadata` file
	    todo!()
	}
    }
}
